package order

import (
	"applet/app/admin/md"
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/e"
	"applet/app/svc"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
	"github.com/360EntSecGroup-Skylar/excelize"
	"github.com/gin-gonic/gin"
	"github.com/tidwall/gjson"
	"strconv"
)

func OrderList(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	list, total := db.GetOrderList(eg, args)
	data := make([]map[string]string, 0)
	if list != nil {
		stateArr := []string{"待制作", "制作中", "制作完成", "烘焙完成", "分拣完成", "已取消"}
		enterpriseIds := make([]int, 0)
		for _, v := range *list {
			enterpriseIds = append(enterpriseIds, v.EnterpriseId)
		}
		enterpriseMap := db.GetEnterpriseMore(eg, enterpriseIds)
		for _, v := range *list {
			v1 := JudgePackageOrdOrdState(&v)
			isCanCancel := "1"
			if v1.State > 0 {
				isCanCancel = "0"
			}
			enterpriseName := ""
			insideEnterpriseName := ""
			_, ok := enterpriseMap[v.EnterpriseId]
			if ok {
				enterpriseName = enterpriseMap[v.EnterpriseId].Name
				insideEnterpriseName = enterpriseMap[v.EnterpriseId].InsideName
			}
			var tmp = map[string]string{
				"buy_phone":       v1.BuyPhone,
				"ord_no":          v1.OrdNo,
				"oid":             utils.Int64ToStr(v1.Oid),
				"id":              utils.IntToStr(v1.Id),
				"amount":          v1.Amount,
				"state_str":       stateArr[v1.State],
				"state":           utils.IntToStr(v1.State),
				"time":            gjson.Get(v1.BuyInfo, "time").String(),
				"date":            gjson.Get(v1.BuyInfo, "date").String(),
				"create_at":       v1.CreateAt.Format("2006-01-02 15:04:05"),
				"is_can_cancel":   isCanCancel,
				"enterprise_name": enterpriseName,
				"inside_name":     insideEnterpriseName,
			}
			data = append(data, tmp)
		}
	}
	state := []map[string]string{
		{"name": "制作中", "value": "1"},
		{"name": "已完成", "value": "4"},
		{"name": "已取消", "value": "5"},
	}
	res := map[string]interface{}{
		"total": total,
		"list":  data,
		"state": state,
	}
	e.OutSuc(c, res, nil)
	return
}

func OrderExport(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	list, _ := db.GetOrderListExport(eg, args)
	xlsx, _ := excelize.OpenFile("./static/template/商品销售明细表.xlsx")

	var data = map[string]map[int64][]map[string]string{}
	if list != nil {
		enterpriseIds := make([]int, 0)
		for _, v := range *list {
			enterpriseIds = append(enterpriseIds, v.EnterpriseId)
		}
		enterpriseMap := db.GetEnterpriseMore(eg, enterpriseIds)

		for _, v := range *list {
			enterpriseName := "-"
			inSideName := "-"
			_, ok := enterpriseMap[v.EnterpriseId]
			if ok {
				enterpriseName = enterpriseMap[v.EnterpriseId].Name
				inSideName = enterpriseMap[v.EnterpriseId].InsideName
			}
			if data[utils.IntToStr(v.EnterpriseId)] == nil {
				data[utils.IntToStr(v.EnterpriseId)] = make(map[int64][]map[string]string)
			}
			if data[utils.IntToStr(v.EnterpriseId)][v.Oid] == nil {
				data[utils.IntToStr(v.EnterpriseId)][v.Oid] = make([]map[string]string, 0)
			}
			//TODO:: 查找 `order_goods`
			var orderGoods []model.OrderGoods
			err := eg.Where("oid = ? and goods_type=?", v.Oid, 0).Find(&orderGoods)
			if err != nil {
				e.OutErr(c, e.ERR_DB_ORM, err)
				return
			}
			for _, vv := range orderGoods {
				tmp := make(map[string]string)
				tmp["date"] = v.CreateAt.Format("2006-01-02")
				tmp["send_date"] = gjson.Get(v.BuyInfo, "date").String()
				tmp["goods_name"] = vv.GoodsTitle
				tmp["inside_name"] = inSideName
				tmp["enterprise_name"] = enterpriseName
				tmp["ord_no"] = vv.OrdNo
				tmp["goods_type"] = utils.IntToStr(vv.GoodsType)
				skuData := make([]md.Sku, 0)
				json.Unmarshal([]byte(vv.Sku), &skuData)
				skuStr := ""
				for _, v1 := range skuData {
					if skuStr != "" {
						skuStr += ";"
					}
					skuStr += v1.Value
				}
				tmp["sku_str"] = skuStr
				tmp["num"] = utils.IntToStr(vv.Num)
				data[utils.IntToStr(v.EnterpriseId)][v.Oid] = append(data[utils.IntToStr(v.EnterpriseId)][v.Oid], tmp)
			}
		}
	}

	sheetIndex := 2         //第一个表是模板表
	xlsx.NewSheet("Sheet1") //新建表格
	xlsx.CopySheet(1, sheetIndex)
	xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(2), &[]interface{}{
		"日期：" + utils.TimeParseStd(args["start_at"]).Format("2006-01-02") + " ~ " + utils.TimeParseStd(args["end_at"]).Format("2006-01-02"),
	})

	j := 4 //表头前三行被占用了，只能从第四行开始
	for k, v := range data {
		var total = 0
		for kk, vv := range v {
			for _, v3 := range vv {
				if v3["ord_no"] == "" {
					v3["ord_no"] = utils.Int64ToStr(kk)
				}
				if utils.StrToInt(v3["goods_type"]) > 0 {
					v3["goods_name"] = "数量不计入汇总-" + v3["goods_name"]
				}
				xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{
					v3["date"],
					v3["ord_no"],
					v3["inside_name"],
					v3["goods_name"],
					v3["sku_str"],
					v3["num"],
				})
				xlsx.SetRowHeight("Sheet1", j, 18)
				if utils.StrToInt(v3["goods_type"]) == 0 {
					total += utils.StrToInt(v3["num"])
				}
				j++
			}
		}

		style, err := xlsx.NewStyle(`{"fill":{"type":"pattern","color":["FFFFCC"],"pattern":1}, "alignment":{"horizontal":"center", "vertical": "center"}}`)
		if err != nil {
			logx.Error(err)
			println("<<<<SelfSupportForSchoolDataStatisticsExport4>>>>>Error:::", err.Error())
			panic(err)
		}
		xlsx.MergeCell("Sheet1", "A"+strconv.Itoa(j), "F"+strconv.Itoa(j)) //合并单元格
		xlsx.SetCellStyle("Sheet1", "A"+strconv.Itoa(j), "F"+strconv.Itoa(j), style)
		xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{
			k + " 汇总： " + utils.IntToStr(total),
		})
		xlsx.SetRowHeight("Sheet1", j, 20)
		j++
	}
	xlsx.DeleteSheet("template") //删除模板表

	c.Header("Content-Type", "application/octet-stream")
	c.Header("Content-Disposition", "attachment; filename="+"订单记录"+".xlsx")
	c.Header("Content-Transfer-Encoding", "binary")
	//回写到web 流媒体 形成下载
	_ = xlsx.Write(c.Writer)
	return
}

// JudgePackageOrdOrdState 判断订单状态
func JudgePackageOrdOrdState(ord *model.Order) *model.Order {
	if ord.State == 5 {
		return ord
	}
	var ordState, oldOrdState int
	oldOrdState = ord.State
	ordState = oldOrdState
	//全部订单
	countAll, err := db.Db.Where("oid =?", ord.Oid).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	//1、判断是否有 `制作中` 有一个就是制作中
	count1, err := db.Db.Where("oid =?", ord.Oid).And("state =?  and goods_type=?", 1, 0).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	if count1 > 0 {
		ordState = 1
	}
	//3、判断是否有 `烘焙中` 要全部制作完成
	count2, err := db.Db.Where("oid =?", ord.Oid).And("state =?", 2).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	if count2 > 0 && count2 == countAll {
		ordState = 2
	}
	//3、判断是否有 `分拣中` 要全部烘焙完成
	count3, err := db.Db.Where("oid =?", ord.Oid).And("state =?", 3).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	if count3 > 0 && count3 == countAll {
		ordState = 3
	}
	//4、判断是否有 `已完成` 要全部已完成
	count4, err := db.Db.Where("oid =?", ord.Oid).And("state =? and goods_type=?", 4, 0).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	if count4 > 0 && count4 == countAll {
		ordState = 4
	}
	if ordState != oldOrdState {
		ord.State = ordState
		_, err2 := db.Db.Where("oid=?", ord.Oid).Cols("state").Update(&ord)
		if err2 != nil {
			return ord
		}
	}
	return ord
}
func OrderTotal(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	where := "1=1"

	if args["phone"] != "" {
		where += " and buy_phone like '%" + args["phone"] + "%'"
	}
	if args["oid"] != "" {
		where += " and oid like '%" + args["oid"] + "%'"
	}
	if args["goods_title"] != "" {
		var orderGoods []model.OrderGoods
		eg.Where("goods_title like ?", "%"+args["goods_title"]+"%").Find(&orderGoods)
		oids := "-1"
		for _, v := range orderGoods {
			oids += "," + utils.Int64ToStr(v.Oid)
		}
		where += " and oid in(" + oids + ")"
	}
	sql := "select SUM(IF(state!=5,amount,0)) as amount,SUM(IF(state!=5,1,0)) as alls,SUM(IF(state=0,1,0)) as wait_do,SUM(IF(state=1,1,0)) as dos,SUM(IF(state=2,1,0)) as baking,SUM(IF(state=3,1,0)) as sorting,SUM(IF(state=4,1,0)) as success,SUM(IF(state=5,1,0)) as cancel from `order` where %s"
	sql = fmt.Sprintf(sql, where)
	nativeString, _ := db.QueryNativeString(eg, sql)
	var res = make([]string, 0)
	for _, v := range nativeString {
		res = append(res, "订单总金额(元):"+v["amount"])
		res = append(res, "总订单数量:"+v["alls"])
		res = append(res, "制作中订单数:"+v["dos"])
		res = append(res, "已完成订单数:"+v["success"])
		res = append(res, "已取消订单数:"+v["cancel"])
	}
	e.OutSuc(c, res, nil)
	return
}

func OrderCancel(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	order = JudgePackageOrdOrdState(order)
	if order.State == 5 {
		e.OutErr(c, 400, e.NewErr(400, "订单已取消"))
		return
	}
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()
	goods := db.GetOrderGoodsByIds(sess, args["oid"])
	if goods != nil {
		for _, v := range *goods {
			if v.GoodsType > 0 {
				continue
			}
			//判断有没有的扣
			all := db.GetOrderGoodsMakeStockAll(sess, v.GoodsId, v.SkuId, 0)
			if all == nil {
				sess.Rollback()
				e.OutErr(c, 400, e.NewErr(400, "订单取消失败"))
				return
			}
			if all.WaitMakeNum < v.Num {
				sess.Rollback()
				e.OutErr(c, 400, e.NewErr(400, "订单已经在制作，取消失败"))
				return
			}
			all.WaitMakeNum -= v.Num
			_, err := sess.Where("id=?", all.Id).Cols("wait_make_num").Update(all)
			if err != nil {
				sess.Rollback()
				e.OutErr(c, 400, e.NewErr(400, "订单取消失败"))
				return
			}
		}
	}
	order.State = 5
	_, err := sess.Where("oid=?", order.Oid).Cols("state").Update(order)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "订单取消失败"))
		return
	}
	_, err = sess.Where("oid=?", order.Oid).Cols("state").Update(&model.OrderGoods{State: 5})
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "订单取消失败"))
		return
	}
	sess.Commit()
	e.OutSuc(c, "success", nil)
	return
}

func OrderDetail(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	order = JudgePackageOrdOrdState(order)
	var enterpriseData model.Enterprise
	eg.Where("id=?", order.EnterpriseId).Get(&enterpriseData)
	stateArr := []string{"待制作", "制作中", "烘焙中", "分拣中", "已完成", "已取消"}
	enterpriseName := enterpriseData.Name
	insideName := enterpriseData.InsideName

	//订单商品
	var orderGoods []model.OrderGoods
	eg.Where("oid=?", args["oid"]).Find(&orderGoods)
	skuIds := make([]int64, 0)
	gids := make([]int64, 0)

	for _, v := range orderGoods {
		skuIds = append(skuIds, v.SkuId)
		gids = append(gids, v.GoodsId)
	}
	//查商品和sku
	goodsMap := db.GetGoodsMore(eg, gids)
	goodsInfo := make([]map[string]interface{}, 0)
	scheme, host, subDomain, moreSubDomain := svc.ImageBucketNew(c)
	var tmpMap interface{}
	var skus []model.Sku
	eg.Table("sku").In("goods_id", gids).Find(&skus)
	for _, v := range orderGoods {
		skuList := make([]model.Sku, 0)
		for _, skuItem := range skus {
			if skuItem.GoodsId == v.GoodsId {
				skuList = append(skuList, skuItem)
			}
		}
		firstNum := utils.IntToStr(v.FirstNum)
		if v.GoodsType > 0 {
			firstNum = ""
		}
		tmp := map[string]interface{}{
			"goods_img":   "",
			"id":          utils.IntToStr(v.Id),
			"goods_type":  utils.IntToStr(v.GoodsType),
			"first_num":   firstNum,
			"sku_id":      utils.Int64ToStr(v.SkuId),
			"goods_id":    utils.Int64ToStr(v.GoodsId),
			"num":         utils.IntToStr(v.Num),
			"amount":      utils.Float64ToStr(utils.StrToFloat64(v.Price) * float64(v.Num)),
			"price":       utils.StrToFloat64(v.Price),
			"sku_str":     "",
			"spe":         "",
			"sku_list":    skuList,
			"goods_title": v.GoodsTitle,
			"memo":        v.DeductMemo,
		}

		skuData := make([]md.Sku, 0)
		json.Unmarshal([]byte(v.Sku), &skuData)
		skuStr := ""
		for _, v1 := range skuData {
			if skuStr != "" {
				skuStr += ";"
			}
			skuStr += v1.Value
		}
		tmp["sku_str"] = skuStr

		// spe
		var spe []map[string]interface{}
		var speTmp []map[string]interface{}
		utils.Unserialize([]byte(goodsMap[v.GoodsId].Spe), &spe)
		for _, itemSpe := range spe {
			tmp1 := make(map[string]interface{})
			tmp1["type"] = "text"
			tmp1["name"] = itemSpe["name"]
			subTmp := make([]map[string]interface{}, 0)
			for i, v2 := range itemSpe["values"].([]interface{}) {
				subTmp = append(subTmp, map[string]interface{}{
					"key":  i,
					"name": v2,
				})
			}
			tmp1["values"] = subTmp
			speTmp = append(speTmp, tmp1)
		}
		tmp["spe"] = speTmp
		// 商品主图
		if goodsMap[v.GoodsId].ImageList != "" {
			utils.Unserialize([]byte(goodsMap[v.GoodsId].ImageList), &tmpMap)
			var imageListUrlTmp []string
			for _, v := range tmpMap.([]interface{}) {
				imageListUrlTmp = append(imageListUrlTmp, svc.ImageFormatWithBucketNew(scheme, host, subDomain, moreSubDomain, v.(string)))
			}
			if imageListUrlTmp != nil {
				tmp["goods_img"] = imageListUrlTmp[0]
			}
		}
		goodsInfo = append(goodsInfo, tmp)
	}

	var res = map[string]interface{}{
		"ord_no":          order.OrdNo,
		"buy_phone":       order.BuyPhone,
		"oid":             utils.Int64ToStr(order.Oid),
		"id":              utils.IntToStr(order.Id),
		"amount":          order.Amount,
		"state_str":       stateArr[order.State],
		"state":           utils.IntToStr(order.State),
		"date":            gjson.Get(order.BuyInfo, "date").String(),
		"buy_name":        gjson.Get(order.BuyInfo, "name").String(),
		"address":         gjson.Get(order.BuyInfo, "address").String(),
		"time":            gjson.Get(order.BuyInfo, "time").String(),
		"create_at":       order.CreateAt.Format("2006-01-02 15:04:05"),
		"enterprise_name": enterpriseName,
		"inside_name":     insideName,
		"goods_info":      goodsInfo,
	}
	e.OutSuc(c, res, nil)
	return
}

func OrderOutput(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	list := db.GetOrderListOutput(eg, args)
	xlsx, _ := excelize.OpenFile("./static/template/商品销售明细表.xlsx")

	var data = map[string]map[int64][]map[string]string{}
	if list != nil {
		enterpriseIds := make([]int, 0)
		for _, v := range *list {
			enterpriseIds = append(enterpriseIds, v.EnterpriseId)
		}
		enterpriseMap := db.GetEnterpriseMore(eg, enterpriseIds)

		for _, v := range *list {
			enterpriseName := "-"
			inSideName := "-"
			_, ok := enterpriseMap[v.EnterpriseId]
			if ok {
				enterpriseName = enterpriseMap[v.EnterpriseId].Name
				inSideName = enterpriseMap[v.EnterpriseId].InsideName
			}
			if data[utils.IntToStr(v.EnterpriseId)] == nil {
				data[utils.IntToStr(v.EnterpriseId)] = make(map[int64][]map[string]string)
			}
			if data[utils.IntToStr(v.EnterpriseId)][v.Oid] == nil {
				data[utils.IntToStr(v.EnterpriseId)][v.Oid] = make([]map[string]string, 0)
			}
			//TODO:: 查找 `order_goods`
			var orderGoods []model.OrderGoods
			err := eg.Where("oid = ? ", v.Oid).Find(&orderGoods)
			if err != nil {
				e.OutErr(c, e.ERR_DB_ORM, err)
				return
			}
			for _, vv := range orderGoods {
				tmp := make(map[string]string)
				tmp["date"] = v.CreateAt.Format("2006-01-02")
				tmp["send_date"] = gjson.Get(v.BuyInfo, "date").String()
				tmp["goods_name"] = vv.GoodsTitle
				tmp["inside_name"] = inSideName
				tmp["enterprise_name"] = enterpriseName
				tmp["ord_no"] = vv.OrdNo
				tmp["goods_type"] = utils.IntToStr(vv.GoodsType)
				skuData := make([]md.Sku, 0)
				json.Unmarshal([]byte(vv.Sku), &skuData)
				skuStr := ""
				for _, v1 := range skuData {
					if skuStr != "" {
						skuStr += ";"
					}
					skuStr += v1.Value
				}
				tmp["sku_str"] = skuStr
				tmp["num"] = utils.IntToStr(vv.Num)
				data[utils.IntToStr(v.EnterpriseId)][v.Oid] = append(data[utils.IntToStr(v.EnterpriseId)][v.Oid], tmp)
			}
		}
	}

	sheetIndex := 2         //第一个表是模板表
	xlsx.NewSheet("Sheet1") //新建表格
	xlsx.CopySheet(1, sheetIndex)
	xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(2), &[]interface{}{
		"日期：" + utils.TimeParseStd(args["start_at"]).Format("2006-01-02") + " ~ " + utils.TimeParseStd(args["end_at"]).Format("2006-01-02"),
	})

	j := 4 //表头前三行被占用了，只能从第四行开始
	for k, v := range data {
		var total = 0
		for kk, vv := range v {
			for _, v3 := range vv {
				if v3["ord_no"] == "" {
					v3["ord_no"] = utils.Int64ToStr(kk)
				}
				if utils.StrToInt(v3["goods_type"]) > 0 {
					v3["goods_name"] = v3["goods_name"] + "-数量不计入汇总"
				}
				xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{
					v3["date"],
					v3["ord_no"],
					v3["inside_name"],
					v3["goods_name"],
					v3["sku_str"],
					v3["num"],
				})
				xlsx.SetRowHeight("Sheet1", j, 18)
				if utils.StrToInt(v3["goods_type"]) == 0 {
					total += utils.StrToInt(v3["num"])
				}
				j++
			}
		}

		style, err := xlsx.NewStyle(`{"fill":{"type":"pattern","color":["FFFFCC"],"pattern":1}, "alignment":{"horizontal":"center", "vertical": "center"}}`)
		if err != nil {
			logx.Error(err)
			println("<<<<SelfSupportForSchoolDataStatisticsExport4>>>>>Error:::", err.Error())
			panic(err)
		}
		xlsx.MergeCell("Sheet1", "A"+strconv.Itoa(j), "F"+strconv.Itoa(j)) //合并单元格
		xlsx.SetCellStyle("Sheet1", "A"+strconv.Itoa(j), "F"+strconv.Itoa(j), style)
		xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{
			k + " 汇总： " + utils.IntToStr(total),
		})
		xlsx.SetRowHeight("Sheet1", j, 20)
		j++
	}
	xlsx.DeleteSheet("template") //删除模板表

	c.Header("Content-Type", "application/octet-stream")
	c.Header("Content-Disposition", "attachment; filename="+"订单记录"+".xlsx")
	c.Header("Content-Transfer-Encoding", "binary")
	//回写到web 流媒体 形成下载
	_ = xlsx.Write(c.Writer)
	return
}
func OrderEdit(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	var tmp md.BuyInfo
	json.Unmarshal([]byte(order.BuyInfo), &tmp)
	tmp.Time = args["time"]
	tmp.Date = args["date"]
	tmp.Phone = args["buy_phone"]
	tmp.Name = args["buy_name"]
	tmp.Address = args["address"]
	order.BuyPhone = args["buy_phone"]
	order.BuyInfo = utils.SerializeStr(tmp)
	eg.Where("id=?", order.Id).Cols("buy_phone,buy_info").Update(order)
	e.OutSuc(c, "success", nil)
	return
}
func OrderDel(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()
	orderGoodsSku := db.GetOrderGoodsById(eg, args["id"])
	i, err2 := eg.Where("id=?", args["id"]).Delete(&model.OrderGoods{})
	if i == 0 || err2 != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "删除失败"))
		return
	}
	//销量扣除
	sql := `UPDATE goods SET sale=sale-%d WHERE id=%d`
	sql = fmt.Sprintf(sql, orderGoodsSku.Num, orderGoodsSku.GoodsId)
	db.QueryNativeStringSess(sess, sql)
	order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) - float64(orderGoodsSku.Num)*utils.StrToFloat64(orderGoodsSku.Price))
	_, err := sess.Where("id=?", order.Id).Cols("amount").Update(order)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	sess.Commit()
	e.OutSuc(c, "success", nil)
	return
}
func OrderEditNum(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	order = JudgePackageOrdOrdState(order)
	if order.State == 5 {
		e.OutErr(c, 400, e.NewErr(400, "订单已取消,不能修改"))
		return
	}
	//if order.State > 0 {
	//	e.OutErr(c, 400, e.NewErr(400, "订单制作中，不能修改"))
	//	return
	//}
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()

	orderGoods := db.GetOrderGoodsByIdSess(sess, args["id"])
	isHasSku := 0
	if orderGoods.SkuId != utils.StrToInt64(args["sku_id"]) {
		//判断有没有的扣
		all := db.GetOrderGoodsMakeStockAll(sess, orderGoods.GoodsId, orderGoods.SkuId, 0)
		if all == nil {
			sess.Rollback()
			e.OutErr(c, 400, e.NewErr(400, "修改失败"))
			return
		}
		if all.WaitMakeNum < orderGoods.Num {
			sess.Rollback()
			e.OutErr(c, 400, e.NewErr(400, "订单已经在制作，取消失败"))
			return
		}
		all.WaitMakeNum -= orderGoods.Num
		_, err := sess.Where("id=?", all.Id).Cols("wait_make_num").Update(all)
		if err != nil {
			sess.Rollback()
			e.OutErr(c, 400, e.NewErr(400, "修改失败"))
			return
		}
		orderGoodsSku := db.GetOrderGoodsBySkuIdSess(sess, args["oid"], args["sku_id"])
		if orderGoodsSku != nil {
			isHasSku = 1
			//先删掉修改的 ，然后改另外的sku
			i, err := sess.Where("id=?", args["id"]).Delete(&model.OrderGoods{})
			if i == 0 || err != nil {
				sess.Rollback()
				e.OutErr(c, 400, e.NewErr(400, "修改失败"))
				return
			}
			//销量扣除
			sql := `UPDATE goods SET sale=sale-%d WHERE id=%d`
			sql = fmt.Sprintf(sql, orderGoods.Num, orderGoods.GoodsId)
			db.QueryNativeStringSess(sess, sql)
			//销量增加
			sqlSecond := `UPDATE goods SET sale=sale+%d WHERE id=%d`
			sqlSecond = fmt.Sprintf(sqlSecond, args["num"], orderGoodsSku.GoodsId)
			db.QueryNativeStringSess(sess, sqlSecond)
			//重新计算价格
			tmp := orderGoodsSku
			oldNum := orderGoodsSku.Num
			orderGoodsSku.Num += utils.StrToInt(args["num"])
			orderGoodsSku.OldNum += utils.StrToInt(args["num"])
			if utils.StrToFloat64(args["price"]) > 0 && utils.StrToFloat64(args["price"]) != utils.StrToFloat64(orderGoodsSku.Price) {
				order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) - float64(oldNum)*utils.StrToFloat64(orderGoodsSku.Price))
				orderGoodsSku.Price = args["price"]
				order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) + float64(orderGoodsSku.Num)*utils.StrToFloat64(orderGoodsSku.Price))
			} else {
				order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) + utils.StrToFloat64(args["num"])*utils.StrToFloat64(orderGoodsSku.Price))
			}
			orderGoodsSku.DeductMemo = args["memo"]
			_, err = sess.Where("id=?", orderGoodsSku.Id).Cols("num,memo,deduct_memo,old_num,price").Update(orderGoodsSku)
			if err != nil {
				sess.Rollback()
				e.OutErr(c, 400, e.NewErr(400, "修改失败"))
				return
			}
			tmp.Num = utils.StrToInt(args["num"])
			err = Total(sess, *tmp)
			if err != nil {
				sess.Rollback()
				e.OutErr(c, 400, e.NewErr(400, "下单失败"))
				return
			}
		} else {
			orderGoods.SkuId = utils.StrToInt64(args["sku_id"])
			_, sku, _ := db.GetMallSkuBySkuId(eg, args["sku_id"], utils.Int64ToStr(orderGoods.GoodsId))
			if sku != nil {
				orderGoods.SkuCode = sku.SkuCode
				orderGoods.Sku = sku.Sku
			}
		}
	}
	if isHasSku == 0 {
		//判断有没有的扣
		all := db.GetOrderGoodsMakeStockAll(sess, orderGoods.GoodsId, orderGoods.SkuId, 0)
		if all == nil {
			sess.Rollback()
			e.OutErr(c, 400, e.NewErr(400, "修改失败"))
			return
		}
		num := utils.StrToInt(args["num"]) - orderGoods.Num
		if num < 0 { //少要了 就总金额 减少
			if all.WaitMakeNum < orderGoods.Num {
				sess.Rollback()
				e.OutErr(c, 400, e.NewErr(400, "订单已经在制作，取消失败"))
				return
			}
			num = orderGoods.Num - utils.StrToInt(args["num"])
			all.WaitMakeNum -= num
			if utils.StrToFloat64(args["price"]) > 0 && utils.StrToFloat64(args["price"]) != utils.StrToFloat64(orderGoods.Price) {
				order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) - float64(orderGoods.Num)*utils.StrToFloat64(orderGoods.Price))
				orderGoods.Price = args["price"]
				order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) + float64(utils.StrToInt(args["num"]))*utils.StrToFloat64(orderGoods.Price))
			} else {
				order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) - float64(num)*utils.StrToFloat64(orderGoods.Price))
			}
			//销量扣除
			sqlSecond := `UPDATE goods SET sale=sale-%d WHERE id=%d`
			sqlSecond = fmt.Sprintf(sqlSecond, num, orderGoods.GoodsId)
			db.QueryNativeStringSess(sess, sqlSecond)
		} else {
			all.WaitMakeNum += num
			if utils.StrToFloat64(args["price"]) > 0 && utils.StrToFloat64(args["price"]) != utils.StrToFloat64(orderGoods.Price) {
				order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) - float64(orderGoods.Num)*utils.StrToFloat64(orderGoods.Price))
				orderGoods.Price = args["price"]
				order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) + float64(utils.StrToInt(args["num"]))*utils.StrToFloat64(orderGoods.Price))
			} else {
				order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) + float64(num)*utils.StrToFloat64(orderGoods.Price))
			}
			//销量增加
			sqlSecond := `UPDATE goods SET sale=sale+%d WHERE id=%d`
			sqlSecond = fmt.Sprintf(sqlSecond, num, orderGoods.GoodsId)
			db.QueryNativeStringSess(sess, sqlSecond)
		}
		_, err := sess.Where("id=?", all.Id).Cols("wait_make_num").Update(all)
		if err != nil {
			sess.Rollback()
			e.OutErr(c, 400, e.NewErr(400, "修改失败"))
			return
		}
		orderGoods.OldNum = orderGoods.Num
		orderGoods.Num = utils.StrToInt(args["num"])
		orderGoods.DeductMemo = args["memo"]
		_, err = sess.Where("id=?", orderGoods.Id).Cols("num,sku_code,sku_id,sku,memo,deduct_memo,old_num,price").Update(orderGoods)
		if err != nil {
			sess.Rollback()
			e.OutErr(c, 400, e.NewErr(400, "修改失败"))
			return
		}
	}
	_, err := sess.Where("id=?", order.Id).Cols("amount").Update(order)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	sess.Commit()
	e.OutSuc(c, "success", nil)
	return
}
func OrderAddNum(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	order = JudgePackageOrdOrdState(order)
	if order.State == 5 {
		e.OutErr(c, 400, e.NewErr(400, "订单已取消,不能修改"))
		return
	}
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()
	num := utils.StrToInt(args["num"])

	orderGoods := db.GetOrderGoodsByIdSess(sess, args["id"])
	orderGoods.OldNum = orderGoods.Num
	orderGoods.Num += utils.StrToInt(args["num"])
	_, err := sess.Where("id=?", orderGoods.Id).Cols("num,sku_code,sku_id,sku,old_num,price").Update(orderGoods)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	orderGoods1 := orderGoods
	orderGoods1.Id = 0
	orderGoods1.GoodsTitle = "(新增)" + orderGoods1.GoodsTitle
	orderGoods1.IsNew = 1
	orderGoods1.GoodsType = 1
	orderGoods1.Num = num
	sess.InsertOne(orderGoods1)
	//判断有没有的扣
	all1 := db.GetOrderGoodsMakeStockAll(sess, orderGoods1.GoodsId, orderGoods1.SkuId, 0)
	if all1 == nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	all1.WaitMakeNum += num
	_, err = sess.Where("id=?", all1.Id).Cols("wait_make_num").Update(all1)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	all := all1
	all.Id = 0
	all.WaitBakingNum = 0
	all.WaitMakeNum = num
	all.WaitSortingNum = 0
	all.GoodsTitle = orderGoods1.GoodsTitle
	all.GoodsType = 1
	_, err = sess.InsertOne(all)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) + float64(num)*utils.StrToFloat64(orderGoods1.Price))
	//销量增加
	sqlSecond := `UPDATE goods SET sale=sale+%d WHERE id=%d`
	sqlSecond = fmt.Sprintf(sqlSecond, num, orderGoods1.GoodsId)
	db.QueryNativeStringSess(sess, sqlSecond)
	_, err = sess.Where("id=?", order.Id).Cols("amount").Update(order)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}

	sess.Commit()
	e.OutSuc(c, "success", nil)
	return
}
func OrderDeductNum(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	order = JudgePackageOrdOrdState(order)
	if order.State == 5 {
		e.OutErr(c, 400, e.NewErr(400, "订单已取消,不能修改"))
		return
	}
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()
	num := utils.StrToInt(args["num"])
	orderGoods := db.GetOrderGoodsByIdSess(sess, args["id"])
	orderGoods.OldNum = orderGoods.Num
	orderGoods.Num -= utils.StrToInt(args["num"])
	_, err := sess.Where("id=?", orderGoods.Id).Cols("num,sku_code,sku_id,sku,old_num,price").Update(orderGoods)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	orderGoods1 := orderGoods
	orderGoods1.Id = 0
	orderGoods1.GoodsTitle = "(减少)" + orderGoods1.GoodsTitle
	orderGoods1.IsNew = 1
	orderGoods1.GoodsType = 2
	orderGoods1.Num = num
	sess.InsertOne(orderGoods1)
	//判断有没有的扣
	all1 := db.GetOrderGoodsMakeStockAll(sess, orderGoods1.GoodsId, orderGoods1.SkuId, 0)
	if all1 == nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	if all1.WaitMakeNum < num {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "订单已经在制作，减少数量失败"))
		return
	}
	all1.WaitMakeNum -= num
	_, err = sess.Where("id=?", all1.Id).Cols("wait_make_num").Update(all1)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	all := all1
	all.Id = 0
	all.WaitBakingNum = 0
	all.WaitMakeNum = num
	all.WaitSortingNum = 0
	all.GoodsTitle = orderGoods1.GoodsTitle
	all.GoodsType = 2
	_, err = sess.InsertOne(all)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) - float64(num)*utils.StrToFloat64(orderGoods1.Price))
	//销量减少
	sqlSecond := `UPDATE goods SET sale=sale-%d WHERE id=%d`
	sqlSecond = fmt.Sprintf(sqlSecond, num, orderGoods1.GoodsId)
	db.QueryNativeStringSess(sess, sqlSecond)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	_, err = sess.Where("id=?", order.Id).Cols("amount").Update(order)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}

	sess.Commit()
	e.OutSuc(c, "success", nil)
	return
}
